#! /bin/bash
# build-pclos-kernels is copyrighted by Vaughan Agrez
# build-pclos-kernels is released under GPL licence (version 2)
# AUTHOR: Vaughan Agrez
# EMAIL: ocilent1@gmail.com


###
# Defines
###
VERSION=0.2.5
NAME="Build PCLinuxOS Kernels"
NAMEL=build-pclinuxos-kernels
NAVER="$NAME ($VERSION)"
KSPEC=../SPECS/kernel-pclos.spec
LOGFILE=$NAMEL.log

# building as root user uses a different RPM tmp dir
if [ $UID = 0 ] ; then
	LOGDIR=/tmp
else
	LOGDIR=../tmp
fi

LOG=$LOGDIR/$LOGFILE

#Messages
SPECWARN="Can not find kernel-pclos.spec file.
You require a correctly named and placed spec file to run this build script.

Exiting the build process"
NOBUILD="You chose not to build any kernels"
FINBUILD="Finished the PCLinuxOS kernel build process"

#kernel descriptions
DESCR_DEF="default i686 kernel (4GB, smp)"
DESCR_LEGACY="i586 kernel for legacy cpu support (1GB lowmem, up)"
DESCR_A64PAE="32bit AMD64 (K8) athlon optimised kernel (PAE, smp) *EXPERIMENTAL"
DESCR_A641GB="32bit AMD64 (K8) athlon optimised kernel (1GB lowmem, smp) *EXPERIMENTAL"
DESCR_RT="i686 kernel with Ingo Molnar's realtime preemption patch (4GB, smp) *EXPERIMENTAL"
DESCR_LINUS="i686 kernel from kernel.org sources with no added patches (4GB, smp)"
DESCR_XEN="i686 32bit Xen hypervisor and guests kernels (PAE, smp) *EXPERIMENTAL"


###
# Functions
###

logit()
{
echo "$(date): $1" >> $LOG
}

spec_def()
{
grep "%define $1" $KSPEC |awk '{ print $3 }'
}

krelease()
{
KBASE=2.6
SUBLEVEL=$(spec_def 'sublevel')
STABREL=$(spec_def 'stabrel')
OCIRELEASE=$(spec_def 'ocirelease')
DEVREL=$(spec_def 'devkernel')
STABPATCH=$(spec_def 'stabpatch')

if test "$STABPATCH" = "1"; then
	if test "$DEVREL" = "1"; then
		KVERSION=$KBASE.$SUBLEVEL.$STABREL.dev$OCIRELEASE
	else
		KVERSION=$KBASE.$SUBLEVEL.$STABREL.tex$OCIRELEASE
	fi
	KLINUS=$KBASE.$SUBLEVEL.$STABREL
else
	if test "$DEVREL" = "1"; then
		KVERSION=$KBASE.$SUBLEVEL.dev$OCIRELEASE
	else
		KVERSION=$KBASE.$SUBLEVEL.tex$OCIRELEASE
	fi
	KLINUS=$KBASE.$SUBLEVEL
fi
}

infobuild()
{
if test "$DIALOGDEF" != "1"; then
	zenity --notification --text $"Currently building the $KERN_VERS kernel." & ZENITY_PID="$!"
fi
}

infokill()
{
if test "$DIALOGDEF" != "1"; then
	if [ -d /proc/$ZENITY_PID ]; then
		kill $ZENITY_PID &>/dev/null
	fi
fi
}

buildcmd()
{	
echo "$1" > $LOGDIR/$KERN_VERS-build.log
logit "$KERN_VERS build started"

if test "$1" = "--target i686"; then
	rpmbuild -ba $KSPEC $1 2>>$LOGDIR/$KERN_VERS-build.log
else
	rpmbuild -bb $KSPEC $1 2>>$LOGDIR/$KERN_VERS-build.log
fi

if [ "$?" != "0" ]; then
	logit "***** WARNING: $KERN_VERS FAILED TO BUILD!*****"
	logit "Please check $LOGDIR/$KERN_VERS-build.log for build errors"
else
	logit "all your $KERN_VERS are belong to us."
fi
infokill
}

selection()
{
if test "$DIALOGDEF" = "1"; then
	dialog --checklist "Please select the kernel you want to build (Kernel Version: $KVERSION)" 0 0 0 \
	"DEFAULT" "$DESCR_DEF" ON \
	"LEGACY" "$DESCR_LEGACY" OFF \
	"RT" "$DESCR_RT" OFF \
	"LINUS" "$DESCR_LINUS" OFF \
	"AMD64-PAE" "$DESCR_A64PAE" OFF \
	"AMD64-1GB" "$DESCR_A641GB" OFF > $LOGDIR/.tempsel
 	
#	"XEN" "$DESCR_XEN" OFF 2> $LOGDIR/.tempsel
	SEL=$(cat $LOGDIR/.tempsel)
	rm -f $LOGDIR/.tempsel
else
	SEL=`zenity --list --checklist --width=700 --height=300 \
	--title "Please select the kernel you want to build (Kernel Version: $KVERSION)" \
	--column="" --column "Kernel type" --column "Description" \
	TRUE "DEFAULT" "$DESCR_DEF" \
	FALSE "LEGACY" "$DESCR_LEGACY" \
	FALSE "RT" "$DESCR_RT" \
	FALSE "LINUS" "$DESCR_LINUS" \
	FALSE "AMD64-PAE" "$DESCR_A64PAE" \
	FALSE "AMD64-1GB" "$DESCR_A641GB"`
#	FALSE "XEN" "$DESCR_XEN"`
fi
}

buildselection()
{	
logit "SELECTED BUILDS: $SEL"
if echo "$SEL" | grep 'DEFAULT' &>/dev/null; then
	KERN_VERS=kernel-$KVERSION
	buildcmd "--target i686"
fi
if echo "$SEL" | grep 'LEGACY' &>/dev/null; then
	KERN_VERS=kernel-legacy-$KVERSION.lgc
	buildcmd "--with legacy --target i586"
fi
if echo "$SEL" | grep 'RT' &>/dev/null; then
	KERN_VERS=kernel-rt-$KVERSION.rt
	buildcmd "--with rt --target i686"
fi
if echo "$SEL" | grep 'AMD64-PAE' &>/dev/null; then
	KERN_VERS=kernel-a64-$KVERSION.a64
	buildcmd "--with a64 --target athlon"
fi
if echo "$SEL" | grep 'AMD64-1GB' &>/dev/null; then
	KERN_VERS=kernel-a64-1GB-$KVERSION.k8l
	buildcmd "--with a641gb --target athlon"
fi
if echo "$SEL" | grep 'LINUS' &>/dev/null; then
	KERN_VERS=kernel-linus-$KLINUS
	buildcmd "--with linus --target i686"
fi
#if echo "$SEL" | grep 'XEN' &>/dev/null; then
#	KERN_VERS=kernel-xen0-$KVERSION.xen0
#	buildcmd "--with xen0 --target i686"
#	KERN_VERS=kernel-xenU-$KVERSION.xenU
#	buildcmd "--with xenU --target i686"
#fi

if test "$SEL" = ""; then
	logit "NO KERNELS SELECTED!"
else
	logit "FINISHED KERNEL BUILD PROCESS"
fi
echo "**************" >> $LOG
echo "" >> $LOG
}

depends()
{
if [ ! -f /usr/bin/zenity ]; then
	echo "*** ATTENTION ***"
	echo "This build script can use Zenity if available"
	echo "Defaulting to using Dialog"
	DIALOGDEF=1
fi

if [ ! -f /usr/bin/dialog ]; then
	echo "*** ATTENTION ***"
	echo "This build script requires dialog (cdialog) to run correctly"
	echo "Please install cdialog to use this script"
	echo "Exiting the build process"
	exit 0
fi

if [ ! -f $KSPEC ]; then
	if test "$DIALOGDEF" = "1"; then
		dialog --title "*** ATTENTION ***" --msgbox "$SPECWARN" 0 0
		exit 0
	else
		zenity --error --title "*** ATTENTION ***" --text $"$SPECWARN"
		exit 0
	fi
fi
}

goodbye()
{
	if test "$SEL" = ""; then
		if test "$DIALOGDEF" = "1"; then
			dialog --title "*** ATTENTION ***" --msgbox "$NOBUILD" 0 0
		else
			zenity --info --title "$NAVER" --text $"$NOBUILD"
		fi
	else
		echo "FINISHED KERNEL BUILD PROCESS"
		echo "EXITING"
	fi
	exit 0	
}

buildhelp()
{
	echo ""
	echo "	USAGE: $0 [--build <kernel-type> | --build-all |"
	echo "	--no-zenity | --version ]"
	echo ""
	echo "	--build <kernel-type>: build only the selected kernel type"
	echo "	  (Avaliable kernel types are: DEFAULT, LEGACY, AMD64-PAE, AMD64-1GB"
	echo "	   RT, LINUS or XEN)"
	echo "	--build-all: build all available kernel types (includes experimental builds)"
	echo "	--no-zenity: disable the use of Zenity dialogues"
	echo "	--version: (-v) show script version information"
	echo "	--help: (-h) show this help message"
	echo ""
	echo "	**Passing no build options also lets you select individual kernel builds**"
	echo ""
	echo ""
}

version()
{
echo ""
echo "$NAME -- Version $VERSION"
echo "AUTHOR: ocilent1"
echo "LICENCE: GPL V2"
echo ""
}

main()
{
depends
krelease
echo "**************" > $LOG
logit "$NAVER"
logit "BASE KERNEL VERSION: $KVERSION"
if test "$SEL" = ""; then
	selection
fi
buildselection
goodbye
}

###
# Main
###

#don't use zenity if X is not running!
if [ -z "$DISPLAY" ] ; then
DIALOGDEF=1
fi

case $1 in
"--build" )
	DIALOGDEF=1
	SEL="`echo $@ | sed -e 's/--build //'`"
	main
	;;
"--build-all" )
	DIALOGDEF=1
	SEL="DEFAULT LEGACY RT AMD64-PAE AMD64-1GB LINUS"
#	SEL="DEFAULT LEGACY RT AMD64-PAE AMD64-1GB LINUS XEN"
	main
	;;
"--no-zenity" )
	DIALOGDEF=1
	main
	;;
"--help" )
	buildhelp
	;;
"-h" )
	buildhelp
	;;
"--version" )
	version
	;;
"-v" )
	version
	;;
"" )
	main
	;;
esac
