###
# Definitions
###
%define major		6
%define sublevel	26
%define stabrel		8
%define ocirelease	3
%define codename	weekday warrior


# Is there a stable patchset? (0 no, 1 yes)
%define stabpatch	1
# Oi you! is this a development kernel? (0 no, 1 yes)
%define devkernel	0

# Don't automaticaly clean up build dirs post building (sometimes useful for debugging)
%define dontclean	1


###
# Lets deal with our options and the resulting kernel 'appends' and names
###
%define bname		kernel
%define kname		%{bname}
%if %stabpatch
	%define kappend	%{stabrel}.tex
%else
	%define kappend	tex
%endif

%if %devkernel
	%define codename **PCLinuxOS Development Kernel**
	%if %stabpatch
		%define kappend	%{stabrel}.dev
	%else
		%define kappend	dev
	%endif
%endif

%define tar_version 	2.%{major}.%{sublevel}
%define patchversion 	%{kappend}%{ocirelease}
%define realrelease 	%{kappend}%{ocirelease}

%define rpmversion 	1
%define rpmrelease 	%mkrel 1
%define realversion 	2.%{major}.%{sublevel}
%define ociversion 	%{realversion}.%{realrelease}
%define patches_ver 	2.%{major}.%{sublevel}.%{patchversion}

%define top_dir_name 	%{kname}-legacy-build
%define build_dir 	${RPM_BUILD_DIR}/%top_dir_name
%define src_dir 	%{build_dir}/linux-%tar_version
%define KVERREL 	%{realversion}.%{realrelease}

# Base .config definition
%define config_ver	i386-pclos

###
# Our various kernel build definitions
###
%define build_legacy	1
%define build_a64	0
%define build_a641gb	0
%define build_rt	0
%define build_linus	0
%define build_xen0	0
%define build_xenU	0


#defines for various build options
%define i586build	0
%define preempt_vol	0
%define preempt_full	0
%define lowmem1gb	0
%define lowmem		0
%define k8ops		0
%define xenbuild	0
%define pae_on		0
%define no_smp		0
%define hz_1000		0

# Base .config definition
%define config_ver	i386-pclos

###
# Our various kernel builds
###
#default kernel summary
%define summary i686 32bit linux kernel (4GB)

# legacy kernel build
%{?_with_legacy: %global build_legacy 1}
%if %build_legacy
%define summary		i586 32bit legacy linux kernel
%define kname 		%{bname}-legacy
%define realrelease 	%{kappend}%{ocirelease}.lgc
%define lowmem		1
%define i586build	1
%define no_smp		0
%endif

# AMD64/Opteron x86 build K8 optimised build (PAE)
%{?_with_a64: %global build_a64 1}
%if %build_a64
%define summary		AMD64 (K8) optimised 32bit linux kernel (PAE)
%define kname 		%{bname}-a64
%define optflags 	-O2 -g -pipe -fomit-frame-pointer -march=athlon64 -finline-functions
%define realrelease 	%{kappend}%{ocirelease}.a64
%define k8ops		1
%define preempt_vol	1
%define pae_on		1
%endif

# AMD64/Opteron x86 build K8 optimised build (1GB lowmem)
%{?_with_a641gb: %global build_a641gb 1}
%if %build_a641gb
%define summary		AMD64 (K8) optimised 32bit linux kernel (1GB lowmen)
%define kname 		%{bname}-a64-1GB
%define optflags 	-O2 -g -pipe -fomit-frame-pointer -march=athlon64 -finline-functions
%define realrelease 	%{kappend}%{ocirelease}.k8l
%define k8ops		1
%define lowmem		1
%define lowmem1gb	1
%define preempt_vol	1
%endif


# i686 kernel build with Ingo Molnar's realtime patch
%{?_with_rt: %global build_rt 1}
%if %build_rt
%define summary		i686 kernel with Ingo Molnar's realtime preemption patch (4GB, smp)
%define kname 		%{bname}-rt
%define realrelease 	%{kappend}%{ocirelease}.rt
%define config_ver	i386-pclos-rt
%define hz_1000		1
%endif

# linus kernel build
%{?_with_linus: %global build_linus 1}
%if %build_linus
%define summary		i686 32bit 'Linus' linux kernel (no extra patching) (4GB)
%define kname 		kernel-linus
	%if %stabpatch
	%define realrelease 	%{stabrel}
	%else
	%define ociversion 	%{realversion}
	%define KVERREL 	%{realversion}
	%endif
%define preempt_vol	1
%endif


# xen0 kernel build
#%{?_with_xen0: %global build_xen0 1}
%if %build_xen0
%define summary		32bit XEN hypervisor (Dom0) linux kernel (PAE)
%define kname 		%{bname}-xen0
%define realrelease 	%{kappend}%{ocirelease}.xen0
%define xenbuild	1
%define pae_on		1
%endif

# xenU kernel build
#%{?_with_xenU: %global build_xenU 1}
%if %build_xenU
%define summary		32bit XEN guest (DomU) linux kernel (PAE)
%define kname 		%{bname}-xenU
%define realrelease 	%{kappend}%{ocirelease}.xenU
%define xenbuild	1
%define pae_on		1
%endif





###
# Define some misc messages
###
%define devmessage ***THIS IS 'A WORK IN PROGRESS' DEVELOPMENT KERNEL -- USE AT YOUR OWN RISK! -- IT IS UNSUPPORTED***

%define warning ***PLEASE NOTE: This kernel is an experimental build type and may cause system instability. USE AT YOUR OWN RISK! -- IT IS UNSUPPORTED***


###
# Generic rpm defines
###
%if ! %devkernel
Summary: 	%{summary}
%else
Summary: 	%{summary} *DEVELOPMENT KERNEL*
%endif
Name: 		%{kname}-%{ociversion}
Version:	%{rpmversion}
Release:	%{rpmrelease}
License: 	GPL
Group:		System/Kernel and hardware
ExclusiveArch:	%{ix86} x86_64 amd64
ExclusiveOS: 	Linux
URL: 		http://www.kernel.org/



###
# Deal with our sources
###

# Do you want the src.rpm to build with no kernel source?
%define build_nosrc 1
%{?_with_nosrc: %global build_nosrc 1}

# Due to nosrc.rpm we should check for and if required download the appropriate kernel sources
%(if [ ! -f %_sourcedir/linux-%{tar_version}.tar.bz2 ]; then cd %_sourcedir/ && wget http://www.kernel.org/pub/linux/kernel/v2.%{major}/linux-%{tar_version}.tar.bz2; fi)

# This is for full SRC RPM
Source0: linux-%{tar_version}.tar.bz2
# This is for stripped SRC RPM
%if %build_nosrc
NoSource: 0
%endif

Source99: build-pclos-kernels.sh
Source100: linux-%{patches_ver}.tar.bz2
# RXS
#Source101: %{patches_ver}.rt.tar.gz


###
# Generic rpm Requires and BuildRequires
###
BuildRoot: %{_tmppath}/%{name}-legacy-build
Provides: module-info, alsa
Provides: kernel = %{ociversion}, %{kname} = %{ociversion}
%if %stabpatch
Provides: kernelbase = 2.%{major}.%{sublevel}.%{stabrel}
%else
Provides: kernelbase = 2.%{major}.%{sublevel}
%endif
Requires: module-init-tools, mkinitrd, bootloader-utils, sysfsutils, hal >= 0.5.9
Requires: kheaders-%{ociversion} >= %{rpmversion}
%if %xenbuild
Requires: xen, xen-hypervisor
%endif
BuildRequires: gcc module-init-tools
Autoreqprov: no

###
# DKMS 3rd party driver Provides:
###
%if !%{build_linus}
Provides: ndiswrapper-kernel = 1.53
# Not provided anymore
#Provides: madwifi-kernel = 0.9.3.2
%endif

###
# Kernel rpm descriptions
###
%description
The kernel package contains the Linux %{ociversion} kernel, the core of your
PCLinux operating system. It is optimised for the i686 cpu arch and supports up to
and including 4GB of system memory (via highmem). It defaults to using the 'CFQ'
scheduler, 'smp' support enabled and is compiled with kernel preemption disabled.

%if %devkernel
%devmessage
%endif

%if %{build_legacy}
%description
This package contains a legacy i586 'up' build of the Linux %{ociversion} kernel
which supports up to and including 896MB of system memory (via lowmem). This kernel
is for maximum cpu compatibility for old processors (eg. Pentium I or early AMD K6
CPUs). It defaults to using the 'CFQ'scheduler and both kernel preemption and 'smp' are
disabled. If you run a modern cpu (eg. dual core CPUs) and wish to maximise your
performance and get full support for your CPU, please install any 'i686' kernel
available via the PCLinuxOS repositories.

%if %devkernel
%devmessage
%endif
%endif

%if %{build_a64}
%description
The kernel package contains a 32bit AMD64 (K8) optimised Linux %{ociversion} kernel,
the core of your PCLinux operating system. It is compiled with kernel optimisations
for the K8 cpu arch using more aggressive gcc optflags, 'smp' support is enabled and
supports up to and including 64GB of system memory (PAE enabled). It defaults to using
the 'CFQ' scheduler and is compiled with voluntary kernel preemption enabled.

%warning

%if %devkernel
%devmessage
%endif
%endif

%if %{build_a641gb}
%description
The kernel package contains a 32bit AMD64 (K8) optimised Linux %{ociversion} kernel,
the core of your PCLinux operating system. It is compiled with kernel optimisations
for the K8 cpu arch using more aggressive gcc optflags, 'smp' support is enabled and
supports up to and including 1GB of system memory (via lowmem). It defaults to using
the 'CFQ' scheduler and is compiled with voluntary kernel preemption enabled.

%warning

%if %devkernel
%devmessage
%endif
%endif

%if %{build_rt}
%description
The kernel package contains a 32bit Linux %{ociversion} kernel that is configured for realtime scheduling capabilities. It includes the current rt patch written by Ingo Molnar, is compiled with realtime preemption and it supports up to and including 64GB of system memory (PAE enabled). It defaults to using the 'CFQ' scheduler, 'SMP' support is enabled and the kernel timer interrupt is set to 1000 HZ.

%warning

%if %devkernel
%devmessage
%endif
%endif

%if %{build_linus}
%description
This kernel package contains a 32bit Linux kernel compiled from original kernel.org sources
with no additional patching or 3rd party drivers. It is compiled for the i686 cpu arch,
'smp' support enabled and it supports up to and including 4GB of system memory. It is
commonly reffered to as a 'plain', 'vanilla', 'linus' or 'kernel.org' kernel. It defaults
to using the 'CFQ' scheduler and is compiled with voluntary kernel preemption enabled.
%endif


%if %{build_xen0}
%description
The kernel-xen0 package contains XEN hypervisor kernel (Dom0) which provides
device services to unprivileged guests. This kernel build has PAE support
enabled.

%warning

%if %devkernel
%devmessage
%endif
%endif

%if %{build_xenU}
%description
The kernel-xenU package contains XEN guests kernel (DomU) which runs in
XEN unprivileged guests VMs.This kernel build has PAE support enabled.

%warning

%if %devkernel
%devmessage
%endif
%endif


###
# kernel source packages:
###
%package -n %{kname}-source-%{ociversion}
Version: %{rpmversion}
Release: %{rpmrelease}
Provides: kernel-source
Provides: kernel-devel
Provides: kheaders-%{ociversion} = %{rpmversion}
Conflicts: %{kname}-doc-%{ociversion}
Conflicts: %{kname}-devel-%{ociversion}
Requires: glibc-devel, ncurses-devel, make, gcc, perl
Summary: The %{kname}-%{ociversion} source code.
Group: Development/Kernel
Autoreqprov: no

%description -n %{kname}-source-%{ociversion}
The kernel-source package contains the source code files for the %{ociversion}
Linux kernel. These source files are needed to build most C programs, since
they depend on the constants defined in the source code. The source
files can also be used to build a custom kernel that is better tuned to
your particular hardware, if you are so inclined (and you know what you're
doing).

%package -n %{kname}-devel-%{ociversion}
Version: %{rpmversion}
Release: %{rpmrelease}
Provides: kheaders-%{ociversion} = 1.%{rpmversion}
Provides: kernel-devel = %{ociversion}
Conflicts: %{kname}-source-%{ociversion}
Requires(pre): %{kname}-%{ociversion}
Requires: glibc-devel, ncurses-devel, make, gcc, perl
Summary: The %{kname}-%{ociversion} kernel headers
Group: Development/Kernel
Autoreqprov: no

%description -n %{kname}-devel-%{ociversion}
This package contains the kernel header files for the %{ociversion} linux kernel.
This is a stripped down version of the full %{kname}-source that only holds the
kernel headers, Makefiles, KConfig files and some needed binaries. This package
should be sufficient to build most external drivers against. If you want to build
your own kernel from source, then you need to install the main %{kname}-source package.



###
# kernel-doc: documentation for the Linux kernel
###
%package -n %{kname}-doc-%{ociversion}
Version: %{rpmversion}
Release: %{rpmrelease}
Summary: Various documentation found in the kernel source
Group: Development/Kernel

%description -n %{kname}-doc-%{ociversion}
This package contains only documentation files from the kernel source. Various
bits of information about the Linux kernel and the device drivers shipped
with it are documented in these files. Install this package if you need a reference to
the options that can be passed to kernel modules at load time. This package is NOT
required if you have the %{kname}-source rpm already installed.


#
# kernel-latest: virtual rpm
#
%if ! %devkernel
%package -n %{kname}-latest
Version: 	%{ociversion}
Release: 	%{rpmrelease}
Summary: 	Virtual rpm for latest %{kname}
Group:   	System/Kernel and hardware
Requires:	%{kname}-%{ociversion}

%description -n %{kname}-latest
This package is a virtual rpm that aims to make sure you always have the
latest %{kname} installed.
%if %devkernel
%devmessage
%endif
%endif


###
# Apply our various patches
###
%prep
%setup -q -n %top_dir_name -c
%setup -q -n %top_dir_name -D -T -a100

cd %{patches_ver}
cd ..

%define patches_dir ../%{patches_ver}/
cd %src_dir


# Add our stable patches (if any)
%if %stabpatch
cp %{patches_dir}/stable-patch/*.patch %{patches_dir}/patches/
%endif

# Some patch shifting for the xen builds
%if %xenbuild
cp %{patches_dir}/xen/*.patch %{patches_dir}/patches/
%endif

# Some patch shifting for the kernel-rt build
%if %build_rt
cp %{patches_dir}/rt/*.patch %{patches_dir}/patches/
%endif

## We don't want any (or very few) patches for the linus kernel
%if %{build_linus}
LC_ALL=C %{patches_dir}/scripts/apply_patches --patch_dir=%{build_dir}/%{patches_ver}/stable-patch/
%else
LC_ALL=C %{patches_dir}/scripts/apply_patches
%endif





###
# Kernel .config modifications for the various kernel builds
###

KERN_CONFIG=%{patches_dir}/configs/%{config_ver}.config

#enable kernel config option
enable_y()
{
perl -pi -e "s!# CONFIG_$1 is not set!CONFIG_$1=y!" $KERN_CONFIG
}

#disable kernel config option
disable()
{
perl -pi -e "s!CONFIG_$1=y!# CONFIG_$1 is not set!" $KERN_CONFIG
perl -pi -e "s!CONFIG_$1=m!# CONFIG_$1 is not set!" $KERN_CONFIG
}


# enable i586 kernel build
%if %i586build
	enable_y M586
	disable M686
%endif


# disable 'smp' support
%if %no_smp
	disable SMP
%endif

# AMD64/Opteron x86 K8 kernel optimisation
%if %k8ops
	disable M686
	enable_y MK8
%endif

# lowmem option (disable highmem)
%if %lowmem
	disable HIGHMEM4G
	enable_y NOHIGHMEM
%endif

# 1GB lowmem option
%if %lowmem1gb
	disable VMSPLIT_3G
	enable_y VMSPLIT_3G_OPT
%endif

# enable voluntary preemption
%if %preempt_vol
	disable PREEMPT_NONE
	enable_y PREEMPT_VOLUNTARY
	enable_y PREEMPT_BKL
%endif

# enable full kernel preemption
%if %preempt_full
	disable PREEMPT_NONE
	enable_y PREEMPT
	enable_y PREEMPT_BKL
%endif

# enable PAE support
%if %pae_on
	disable HIGHMEM4G
	enable_y HIGHMEM64G
%endif

%if %hz_1000
	disable HZ_250
	enable_y HZ_1000
%endif

# common xen support modifications
%if %xenbuild
	#enable
	enable_y X86_XEN
	#disable
	for i in X86_PC CPU_FREQ SCHED_MC TCG_TPM SYNCLINK MICROCODE SOUND_PRIME; do
		disable $i
	done
%endif

# xen hypervisor kernel
%if %build_xen0
	enable_y XEN_PRIVILEGED_GUEST
%endif

# xen guest kernel
%if %build_xenU
	#enable
	enable_y XEN_UNPRIVILEGED_GUEST
	#disable
	for i in XEN_DISABLE_SERIAL NDISWRAPPER BCM4400 ARCNET HP100 B44 TOSHIBA ACERHK; do
		disable $i
	done
%endif



###
# Misc bits and pieces
###

# make sure the kernel has the sublevel we know it has...
LC_ALL=C perl -p -i -e "s/^SUBLEVEL.*/SUBLEVEL = %{sublevel}/" Makefile

# get rid of unwanted files
find . -name '*~' -o -name '*.orig' -o -name '*.append' |xargs rm -f



###
# Prepare our kernel for building
###
%build

# common target directories
%define _kerneldir /usr/src/linux-%{KVERREL}
%define _bootdir /boot
%define _modulesdir /lib/modules

# directoriy definitions needed for building
%define temp_root %{build_dir}/temp-root
%define temp_source %{temp_root}%{_kerneldir}
%define temp_boot %{temp_root}%{_bootdir}
%define temp_modules %{temp_root}%{_modulesdir}


# prepare our 3rd party additions
%if ! %{build_linus}
cd %src_dir/3rdparty
perl mkbuild.pl
%endif

verscode()
{
LC_ALL=C perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = .%{realrelease}/" Makefile
LC_ALL=C perl -p -i -e "s/^NAME.*/NAME =%{codename}/" Makefile
}

# set up our kernel name and release
cd %src_dir
make mrproper
%if ! %{build_linus}
# make sure kernel EXTRAVERSION and NAME say what we want it to say
verscode
%endif

#Tempory workaround for "make menuconfig" build problems caused by
#stripping out s390 arch files from sources
LC_ALL=C perl -p -i -e "s/^source/### source/" drivers/crypto/Kconfig

# fetch our kernel .config
cp %{patches_dir}/configs/%{config_ver}.config .config



make oldconfig

#copy our real kernel .config to defconfig
#cp -f .config arch/i386/defconfig
cp -f .config arch/x86/configs/i386_defconfig

###
# Build our kernel
###
make all
# start installing stuff
install -d %{temp_boot}
install -m 644 System.map %{temp_boot}/System.map-%{KVERREL}
install -m 644 .config %{temp_boot}/config-%{KVERREL}
%if %xenbuild
cp -f vmlinuz %{temp_boot}/vmlinuz-%{KVERREL}
%else
cp -f arch/x86/boot/bzImage %{temp_boot}/vmlinuz-%{KVERREL}
%endif
# modules
install -d %{temp_modules}/%{KVERREL}
make INSTALL_MOD_PATH=%{temp_root} KERNELRELEASE=%{KVERREL} modules_install

# save our Module.symvers and System.map from the ravages of mrproper
for i in Module.symvers System.map; do
	cp $i ../
done

# create file list for rpm package
output=../kernel_files.%{KVERREL}
echo "%defattr(-,root,root)" > $output
echo "%{_bootdir}/config-%{KVERREL}" >> $output
echo "%{_bootdir}/vmlinuz-%{KVERREL}" >> $output
echo "%{_bootdir}/System.map-%{KVERREL}" >> $output
echo "%dir %{_modulesdir}/%{KVERREL}/" >> $output
echo "%{_modulesdir}/%{KVERREL}/kernel" >> $output
echo "%{_modulesdir}/%{KVERREL}/modules.*" >> $output

###
# Prepare the kernel sources for packaging
###
make mrproper
%if ! %{build_linus}
# make sure kernel EXTRAVERSION and NAME say what we want it to say
verscode
%endif


cp %{patches_dir}/configs/%{config_ver}.config .config


make oldconfig
make prepare
make scripts

# raise our Module.symvers and System.map from the dead
for i in Module.symvers System.map; do
	mv ../$i .
done

#copy our real kernel .config to defconfig
cp -f .config arch/x86/configs/i386_defconfig

###
# Install the beast
###
%install

cd %src_dir
# Directories definition needed for installing
%define target_source %{buildroot}/%{_kerneldir}
%define target_boot %{buildroot}%{_bootdir}
%define target_modules %{buildroot}%{_modulesdir}

# We want to be able to test several times the install part
rm -rf %{buildroot}
cp -a %{temp_root} %{buildroot}

# create directories infastructure
install -d %{target_source}

tar cf - . | tar xf - -C %{target_source}
chmod -R a+rX %{target_source}

# we remove all the source files that we need
for i in arm cris mips mn10300 parisc ppc64 s390 sh sparc sparc64 arm26 h8300 m68knommu v850 m32r ppc alpha ia64 m68k frv xtensa powerpc avr32 blackfin; do
	rm -rf %{target_source}/arch/$i
	rm -rf %{target_source}/include/asm-$i
done

#save the auto.conf and kernel.release files from certain death
mv %{target_source}/include/config/auto.conf ../
mv %{target_source}/include/config/kernel.release ../

# remove config split dir
rm -rf %{target_source}/include/config/*

#restore auto.conf file
mv ../auto.conf %{target_source}/include/config/
mv ../kernel.release %{target_source}/include/config/

# delete other misc files
rm -f %{target_source}/{.config.old,.config.cmd,.mailmap,.missing-syscalls.d}
rm -rf %{target_source}/arch/i386
#rm -rf %{target_source}/samples

%if %xenbuild
rm -f %{target_source}/include/.asm-ignore
rm -f %{target_source}/vmlinux-stripped
rm -f %{target_source}/vmlinuz
%endif

# gzipping modules
find %{target_modules} -name "*.ko" | xargs gzip -9
for i in %{target_modules}/*; do
  rm -f $i/build $i/source $i/modules.*
done

# we really need the depmod -ae here to make sure
pushd %{target_modules}
for i in *; do
	/sbin/depmod -u -ae -b %{buildroot} -r -F %{target_boot}/System.map-$i $i
	echo $?
done

#create our module descriptions
for i in *; do
	pushd $i
	echo "Creating module.description for $i"
	modules=`find . -name "*.ko.gz"`
	echo $modules | xargs /sbin/modinfo \
	| perl -lne 'print "$name\t$1" if $name && /^description:\s*(.*)/; $name = $1 if m!^filename:\s*(.*)\.k?o!; $name =~ s!.*/!!' > modules.description
	popd
done
popd



###
# Make kernel-devel rpm file and dir list
###
cd %{build_dir}
# deal with dirs first
kheaders=kernel_headers.%{KVERREL}
echo "%defattr(-,root,root)" > $kheaders
#for i in drivers fs kernel lib net security sound arch/x86 arch/um arch/x86_64; do
for i in drivers fs kernel lib net security sound arch/x86 arch/um; do
	find linux-%{realversion}/$i -type d -print >> $kheaders
done

# strip/reanme
perl -pi -e 's!linux-%{realversion}!%dir linux-%{realversion}!' kernel_headers.%{KVERREL}

# header files
for i in lib kernel drivers/char drivers/media/video drivers/md arch/x86; do
	find linux-%{realversion}/$i -name *.h -type f -print >> $kheaders
done

# makefiles and Kconfigs
for i in drivers fs block crypto init ipc lib kernel mm security sound net arch/x86; do
	find linux-%{realversion}/$i -name Makefile -type f -print >> $kheaders
	find linux-%{realversion}/$i -name Kconfig -type f -print >> $kheaders
done

# strip/rename
perl -pi -e 's!linux-%{realversion}!%{_kerneldir}!' kernel_headers.%{KVERREL}

# some extra required bitsn' peices....
for i in kernel/asm-offsets.c Makefile_32.cpu Kconfig.cpu configs/i386_defconfig Kconfig.debug; do
	echo "%{_kerneldir}/arch/x86/$i" >> $kheaders
done

# we need a copy of driver core for external mac80211 builds
echo "%{_kerneldir}/drivers/base/core.c" >> $kheaders



###
# Its rude if you don't clean up after yourself!
###
%if %dontclean
%clean
rm -rf %{buildroot}
%else
%clean
rm -rf %{buildroot}
rm -rf %{build_dir}
%endif



###
# Pre and post install scripts
###
%pre

%preun
/sbin/installkernel -R %{KVERREL}
exit 0

%post

%if %xenbuild
%if %build_xen0
###
# Add basic grub menu support for Xen0 kernels
##

#we don't want any grub entry added or any vmlinuz links to be created
/sbin/installkernel -NL %{KVERREL}
#get grub root (hdx,x) from /boot/grub/install.sh
GRUBROOT=$(grep root /boot/grub/install.sh |sed 's|root ||')
#get root partition (/dev/hdxx) from /etc/fstab
PARTROOT=$(grep " / " /etc/fstab | awk '{ print $1 }')
#add our own grub menu entry for Xen Dom0
echo "
title XEN Dom0 %{KVERREL}
kernel $GRUBROOT/boot/xen.gz dom0_mem=262144
module $GRUBROOT/boot/vmlinuz-%{KVERREL} root=$PARTROOT ro
module $GRUBROOT/boot/initrd-%{KVERREL}.img" >> /boot/grub/menu.lst
%endif
%if %build_xenU
#we don't want any grub entry added or any vmlinuz links to be created
/sbin/installkernel -NL %{KVERREL}
%endif
%else
/sbin/installkernel -S %{KVERREL}
%endif

%postun
/sbin/kernel_remove_initrd %{KVERREL}

#cleanup afer oursleves
rm -rf /lib/modules/%{KVERREL}
#dkms: if using dkms, get rid of old dkms kernel baggage
if [ -d /var/lib/dkms ]; then
	find /var/lib/dkms -name kernel-%{KVERREL}-i586 -type l -print | xargs /bin/rm -rf
	find /var/lib/dkms -name %{KVERREL} -type d -print | xargs /bin/rm -rf
fi


%post -n %{kname}-source-%{ociversion}
cd /usr/src
rm -f linux
ln -snf linux-%{KVERREL} linux

# Please note: draktools is creating nasty soft links pointing to
# the incorrect kernel header dirs at kernel install time.
# We need to delete them here first, before re-creating the correct links.
for i in /lib/modules/%{KVERREL}/{build,source}; do
	if [ -L $i ]; then
		rm -f $i
	fi
done

# we need to create /build only when there is a source tree.
for i in /lib/modules/%{KVERREL}; do
	if [ -d $i ]; then
	ln -sf /usr/src/linux-%{KVERREL} $i/build
	ln -sf /usr/src/linux-%{KVERREL} $i/source
	fi
done

%postun -n %{kname}-source-%{ociversion}
if [ -L /usr/src/linux ]; then
    if [ -L /usr/src/linux -a `ls -l /usr/src/linux 2>/dev/null| awk '{ print $11 }'` = "linux-%{KVERREL}" ]; then
	[ $1 = 0 ] && rm -f /usr/src/linux
    fi
fi
# we need to delete <modules>/build at unsinstall
for i in /lib/modules/%{KVERREL}/{build,source}; do
	if [ -L $i ]; then
		rm -f $i
	fi
done
exit 0

%post -n %{kname}-devel-%{ociversion}
cd /usr/src
rm -f linux
ln -snf linux-%{KVERREL} linux

# Please note: draktools is creating nasty soft links pointing to
# the incorrect kernel header dirs at kernel install time.
# We need to delete them here first, before re-creating the correct links.
for i in /lib/modules/%{KVERREL}/{build,source}; do
	if [ -L $i ]; then
		rm -f $i
	fi
done

# Now re-create the correct links
for i in /lib/modules/%{KVERREL}; do
	if [ -d $i ]; then
	ln -sf /usr/src/linux-%{KVERREL} $i/build
	ln -sf /usr/src/linux-%{KVERREL} $i/source
	fi
done


%postun -n %{kname}-devel-%{ociversion}
if [ -L /usr/src/linux ]; then
    if [ -L /usr/src/linux -a `ls -l /usr/src/linux 2>/dev/null| awk '{ print $11 }'` = "linux-%{KVERREL}" ]; then
	[ $1 = 0 ] && rm -f /usr/src/linux
    fi
fi
# we need to delete <modules>/{build,source} at unsinstall
for i in /lib/modules/%{KVERREL}/{build,source}; do
	if [ -L $i ]; then
		rm -f $i
	fi
done
exit 0




###
# Package lists
###
%files -f kernel_files.%{KVERREL}

# full source
%files -n %{kname}-source-%{ociversion}
%defattr(-,root,root)
%dir %{_kerneldir}
%dir %{_kerneldir}/arch
%dir %{_kerneldir}/include
%{_kerneldir}/.config
%{_kerneldir}/.gitignore
%{_kerneldir}/COPYING
%{_kerneldir}/CREDITS
%{_kerneldir}/Documentation
%{_kerneldir}/MAINTAINERS
%{_kerneldir}/Makefile
%{_kerneldir}/README
%{_kerneldir}/REPORTING-BUGS
%{_kerneldir}/Kbuild
%{_kerneldir}/Module.symvers
%{_kerneldir}/System.map
%{_kerneldir}/arch/x86
%{_kerneldir}/arch/.gitignore
%{_kerneldir}/arch/Kconfig
%{_kerneldir}/arch/um
%{_kerneldir}/block
%{_kerneldir}/crypto
%{_kerneldir}/drivers
#%{_kerneldir}/firmware
%{_kerneldir}/fs
%{_kerneldir}/init
%{_kerneldir}/ipc
%{_kerneldir}/kernel
%{_kerneldir}/lib
%{_kerneldir}/mm
%{_kerneldir}/net
%{_kerneldir}/samples
%{_kerneldir}/security
%{_kerneldir}/scripts
%{_kerneldir}/sound
%{_kerneldir}/usr
%{_kerneldir}/virt
%{_kerneldir}/include/Kbuild
%{_kerneldir}/include/acpi
%{_kerneldir}/include/asm
%{_kerneldir}/include/asm-generic
%{_kerneldir}/include/asm-x86
#%{_kerneldir}/include/asm-x86_64
%{_kerneldir}/include/asm-um
%{_kerneldir}/include/config
%{_kerneldir}/include/crypto
%{_kerneldir}/include/keys
%{_kerneldir}/include/linux
%{_kerneldir}/include/math-emu
%{_kerneldir}/include/net
%{_kerneldir}/include/pcmcia
%{_kerneldir}/include/scsi
%{_kerneldir}/include/sound
%{_kerneldir}/include/video
%{_kerneldir}/include/media
%{_kerneldir}/include/mtd
%{_kerneldir}/include/rxrpc
%{_kerneldir}/include/rdma
%{_kerneldir}/include/xen
#%{_kerneldir}/include/prism2
#%{_kerneldir}/include/wlan
%if %{xenbuild}
%{_kerneldir}/include/xen
%endif
%if !%{build_linus}
%{_kerneldir}/3rdparty
#%{_kerneldir}/include/prism2
#%{_kerneldir}/include/wlan
%endif

# kernel-devel
%files -n %{kname}-devel-%{ociversion} -f kernel_headers.%{KVERREL}
%defattr(-,root,root)
%dir %{_kerneldir}
%dir %{_kerneldir}/block
%dir %{_kerneldir}/crypto
%dir %{_kerneldir}/include
%dir %{_kerneldir}/init
%dir %{_kerneldir}/ipc
%dir %{_kerneldir}/mm
%dir %{_kerneldir}/arch
%dir %{_kerneldir}/virt
%{_kerneldir}/.config
%{_kerneldir}/COPYING
%{_kerneldir}/CREDITS
%{_kerneldir}/MAINTAINERS
%{_kerneldir}/Makefile
%{_kerneldir}/README
%{_kerneldir}/REPORTING-BUGS
%{_kerneldir}/Kbuild
%{_kerneldir}/Module.symvers
%{_kerneldir}/System.map
%{_kerneldir}/scripts
%{_kerneldir}/include/Kbuild
%{_kerneldir}/include/acpi
%{_kerneldir}/include/asm
%{_kerneldir}/include/asm-generic
%{_kerneldir}/include/asm-x86
%{_kerneldir}/include/config
%{_kerneldir}/include/crypto
%{_kerneldir}/include/keys
%{_kerneldir}/include/linux
%{_kerneldir}/include/math-emu
%{_kerneldir}/include/net
%{_kerneldir}/include/pcmcia
%{_kerneldir}/include/scsi
%{_kerneldir}/include/sound
%{_kerneldir}/include/video
%{_kerneldir}/include/media
%{_kerneldir}/include/rxrpc
%{_kerneldir}/include/rdma
%{_kerneldir}/include/xen
#%{_kerneldir}/firmware
#%{_kerneldir}/include/prism2
#%{_kerneldir}/include/wlan
%if %{xenbuild}
%{_kerneldir}/include/xen
%endif
%if !%{build_linus}
#%{_kerneldir}/include/prism2
#%{_kerneldir}/include/wlan
%endif

#kernel documentation
%files -n %{kname}-doc-%{ociversion}
%defattr(-,root,root)
%{_kerneldir}/Documentation

# virtual kernel-latest
%if ! %devkernel
%files -n %{kname}-latest
%defattr(-,root,root)
%endif


%if !%{build_linus}

%changelog
* Tue Jan 20 2009 KDulcimer <kdulcimer@gmail.com> 2.6.26.8.tex3
- build for TinyMe
- Trim changelog

* Wed Oct 22 2008 Gettinther <gettinther@gmail.com> 2.6.26.7.tex1
- Add wireless patch from Linville tree
- Add compat patches to support updated wireless
